#include "common.h"
#include "debug.h"

int ena_smenu (const int style) {
	struct s_result serresult;
	struct s_result rxpresult;
	char *atcommand;
	char *max_entries;
	char str_code[32];
	
	if (style == 1) {
		printf("\nEnabling Service Menu style 1: Full Service Menu\n");
	} else if (style == 2) {
		printf("\nEnabling Service Menu style 2: Network / Audio monitor\n");
	} else if (style == 0) {
		printf("\nDisabling Service Menu...\n");
		serresult = ser_send_ack("AT+CPBS=SM");
		serresult = ser_send_ack("AT+CPBS?");
		rxpresult = rxp(",(\\d+)$", serresult.result[0]);
		atcommand = calloc(8 + strlen(rxpresult.result[1]), sizeof(char));
		sprintf(atcommand, "AT+CPBW=%s", rxpresult.result[1]);
	    serresult = ser_send_ack(atcommand);
	    free(atcommand);
		if (!strncmp(serresult.result[0], "OK", 2)) {
			printf("Entry removed from SIM phonebook!\n");
			printf("Switch off and on phone to complete disabling of the service menu!\n");
		} else {
			printf("Phone did not return 'OK' while clearing out the last SIM-phonebook entry!\n");
			printf("Please remove the 'SL45c' entry from your SIM-book manually!\n");
		}
		return 0;
	} else {
		printf("\nIncorrect Service Menu activation style specified!\n");
		return 0;
	}
	
	/* Select SIM-Stored phonebook */
	serresult = ser_send_ack("AT+CPBS=SM");
	
	/* Find out the maximum ammount of numbers we can store */
	serresult = ser_send_ack("AT+CPBS?");
    rxpresult = rxp(",(\\d+)$", serresult.result[0]);
	max_entries = rxpresult.result[1];
	pmesg(100, "Max %d numbers in SIM phonebook\n", atoi(rxpresult.result[1]));
	
	/* Check if there's data at the 'maxnums' position! */
	atcommand = calloc(8 + strlen(rxpresult.result[1]), sizeof(char));
	sprintf(atcommand, "AT+CPBR=%s", rxpresult.result[1]);
	serresult = ser_send_ack(atcommand);
	free(atcommand);

	/* See if there is data, and if there is, check if it's ours. */
	rxpresult = rxp("(CPBR:)", serresult.result[0]);
	if ( (rxpresult.result[1]!=NULL) && (strncmp(rxpresult.result[1], "CPBR:", 5)==0) ) {
		printf("Found entry at last SIM-position ... ");
		rxpresult = rxp("\"(!SL45c!)\"$", serresult.result[0]);
		if ( (rxpresult.result[1]!=NULL) && (strncmp(rxpresult.result[1], "!SL45c!", 7)==0) ) {
			printf("and it belongs to us! Continue!\n");
		} else {
			printf("but it is *not* ours.\n");
			printf("Please use option --smenu 0 to erase this entry!\n");
			printf("Do not forget to check first. It might hold important data!\n");
			return 0;
		}
	} else {
		printf("Last entry on SIM is empty! Nice!\n");
	}
		
	/* Find IMSI number & calculate servicemenu code */
	serresult = ser_send_ack("AT+CIMI");
	siemens_code(serresult.result[0], str_code, style);
	pmesg(100, "Code for IMSI '%s' = '%s'\n", serresult.result[0], str_code);

	/* Build AT-command to write code */
	atcommand = calloc(80, sizeof(char));
	sprintf(atcommand, "AT+CPBW=%s,\"%s\",129,\"!SL45c!\"", max_entries, str_code);
	serresult = ser_send_ack(atcommand);
	pmesg(100, "Phone returned '%s'\n", serresult.result[0]);
	
	if (!strncmp(serresult.result[0], "OK", 2)) {
		printf("OK! Now switch off and on your phone, enter the menu and press * or #\n");
	} else {
		printf("Phone did not return OK after SIM-Entry write command!\n");
		printf("Switch the phone off and on and try pressing * or # in the menu.\n");
	}		
	return 0;						
}

void write_memory (const char *type, const int space) {
	int fd;
	unsigned char buf;
	char *hex;
	char *filename;
	int counter = 0;

	struct stat statinfo;

	char *atcommand;
	int datablock_counter = 0;
	int last_block = 0;

	/* Do we use default filenames, or did user specify one? */
	if (strcmp(tasks.infile, "sl45c") == 0) { /* We need to construct a filename here ... */
		filename = malloc(sizeof(char) * (strlen(tasks.infile)+4));
		strncpy(filename, tasks.infile, strlen(tasks.infile));
		strncpy(filename+strlen(filename), ".", 1);
		strncpy(filename+strlen(filename), type, strlen(type));
	} else { /* We take user input for granted ... */
		filename = malloc(sizeof(char) * strlen(tasks.infile));
		strncpy(filename, tasks.infile, strlen(tasks.infile));
	}

	/* open the file, and calculate how many data_blocks we get */
	fd = open(filename, O_RDONLY);
	if (fd <= 0) {
		printf("\nUnable to open %s? (%s)\n", filename, strerror(errno));
		exit (0);
	}
	fstat(fd, &statinfo);
	/* 175 = size of one data_block = 350 characters in hex */
	last_block = ceil( (float)statinfo.st_size / (float)175 );

	pmesg(100, "Filesize: %d - Blocks: %d\n", (int)statinfo.st_size, last_block);

	/* Here we do the opposite as with read_memory, here we take a binary
	 * file and byte by byte convert it to it's hex equivalent. Then we pump
	 * the hex data to the phone */

	printf ("\nWriting to %s-type memory position %d from %s ...\n", type, space, filename);
	hex = malloc(sizeof(char) * 351);
	while (!(read(fd, &buf, 1) <= 0)) {
		sprintf(hex+counter, "%02x", (unsigned int)buf);
		counter+=2;
		if (counter >= 350) {
			datablock_counter++;
			printf("Block %d of %d\r", datablock_counter, last_block);
			atcommand = calloc(22,sizeof(char));
			sprintf(atcommand, "AT^SBNW=\"%s\",%d,%d,%d", type, space, datablock_counter, last_block);
			non_block_ser_send_ack(atcommand, 1);  /* Send Write Request */
			non_block_ser_send_ack(hex, 0);        /* Send Hex Data */
			non_block_ser_send_ack("\32", 0);
			counter = 0;
		}
	}
	datablock_counter++;
	printf("Block %d of %d\n", datablock_counter, last_block);
	atcommand = calloc(22,sizeof(char));
	sprintf(atcommand, "AT^SBNW=\"%s\",%d,%d,%d", type, space, datablock_counter, last_block);
	non_block_ser_send_ack(atcommand, 1);
	non_block_ser_send_ack(hex, 0);
	non_block_ser_send_ack("\32", 0);
	free(atcommand);
	free(hex);
}

void read_memory_vcf (const int start, const int end) {
	struct s_result serresult;
	struct s_vcard *vcard_result;
	int displayedCSVheader = 0;
	int counter = 0;
	int elemc = 0;
	unsigned int charc = 0;
	char *atcommand;
	char *hex;

	char *vcard_storage;
	char *tmp_char;

	tmp_char = calloc(1, sizeof(char)); /* Storage for 1 character :) */

	for (counter = start; counter <= end; counter++) {
		atcommand=calloc(16, 1);
		snprintf(atcommand, 16, "AT^SBNR=vcf,%d", counter);
		serresult = ser_send_ack(atcommand);

		vcard_storage = calloc(SIZE_VCARD, 1);
		hex = malloc(sizeof(char) * 2);
		for (elemc = 0; elemc < serresult.elemcount; elemc++) {
			if (!strstr(serresult.result[elemc], "^SBNR") && !strstr(serresult.result[elemc], "OK")) {
				for (charc = 0; charc < strlen(serresult.result[elemc]); charc+=2) {
					strncpy(hex, serresult.result[elemc]+charc, 2);
					snprintf(tmp_char, 2, "%c", (int)strtol(hex, NULL, 16));
					strncat(vcard_storage, tmp_char, 1);
				}
			}
		}

		if (strlen(vcard_storage) > 0) {
			vcard_result = parse_vcard(vcard_storage);
			if (tasks.vcard_plain) { /* Normal Output */
				printf("Memory Position: %d/%d\n", counter, end);
				printf("Modified    : %02d-%02d-%02d at %02d:%02d:%02d\n", vcard_result->last_mod.day,
					vcard_result->last_mod.month, vcard_result->last_mod.year,
					vcard_result->last_mod.hour, vcard_result->last_mod.min, vcard_result->last_mod.sec);
				printf("Name        : %s %s\n", vcard_result->firstname, vcard_result->lastname);
				printf("Organisation: %s\n", vcard_result->organisation);
				printf("Street      : %s\n", vcard_result->street);
				printf("Postal      : %s\n", vcard_result->postal);
				printf("City        : %s\n", vcard_result->city);
				printf("Country     : %s\n", vcard_result->country);
				printf("Home        : %s\n", vcard_result->home);
				printf("Cell        : %s\n", vcard_result->cell);
				printf("Work        : %s\n", vcard_result->work);
				printf("Fax         : %s\n", vcard_result->fax);
				printf("Email       : %s\n", vcard_result->email);
				printf("Website     : %s\n", vcard_result->url);
			} else { /* CSV Output */
				if (!displayedCSVheader) {
					printf("day,month,year,hour,min,sec,firstname,lastname,organisation,street,postal,city,country,home,cell,work,fax,email,url\n");
					displayedCSVheader = 1;
				}
				printf("\"%02d\",\"%02d\",\"%02d\",\"%02d\",\"%02d\",\"%02d\",", vcard_result->last_mod.day,
					vcard_result->last_mod.month, vcard_result->last_mod.year,
					vcard_result->last_mod.hour, vcard_result->last_mod.min, vcard_result->last_mod.sec);
				printf("\"%s\",", vcard_result->firstname);
				printf("\"%s\",", vcard_result->lastname);
				printf("\"%s\",", vcard_result->organisation);
				printf("\"%s\",", vcard_result->street);
				printf("\"%s\",", vcard_result->postal);
				printf("\"%s\",", vcard_result->city);
				printf("\"%s\",", vcard_result->country);
				printf("\"%s\",", vcard_result->home);
				printf("\"%s\",", vcard_result->cell);
				printf("\"%s\",", vcard_result->work);
				printf("\"%s\",", vcard_result->fax);
				printf("\"%s\",", vcard_result->email);
				printf("\"%s\"\n", vcard_result->url);
			}
			free(vcard_result);
		} else {
			pmesg(100, "Memory position %d empty!\n", counter);
		}

		free(vcard_storage);
		free(atcommand);
	}
}

/* read_memory should be used for memory types bmp and mid, vcf and vcs
 * have their own handler because these formats need to be parsed first... */
void read_memory (const char *type, const int space) {
	struct s_result serresult;
	int counter = 0;
	unsigned int charc = 0;
	int bytecount = 0;
	char *hex;
	char *atcommand;
	FILE *fd;
	char *filename;

	/* Do we use default filenames, or did user specify one? */
	if (strcmp(tasks.outfile, "sl45c") == 0) {  /* We need to construct a filename here ... */
		filename = malloc(sizeof(char) * (strlen(tasks.outfile)+4));
		strncpy(filename, tasks.outfile, strlen(tasks.outfile));
		strncpy(filename+strlen(filename), ".", 1);
		strncpy(filename+strlen(filename), type, strlen(type));
	} else { /* We take user input for granted ... */
		filename = malloc(sizeof(char) * strlen(tasks.outfile));
		strncpy(filename, tasks.outfile, strlen(tasks.outfile));
	}

	fd = fopen(filename,"w");
	if ((int)fd <= 0) {
		printf("\nUnable to open %s? (%s)\n", filename, strerror(errno));
		exit (0);
	}

	printf ("\nDumping %s-type memory position %d to %s ...\n", type, space, filename);
	/* Construct correct AT-command based on 'type' and 'space' */
	atcommand = malloc(13);
	sprintf(atcommand, "AT^SBNR=%s,%d", type, space);

	/* Allocate space for storing hexadecimal values, send the atcommand and process
	 * the returned strings. We filter out all other crap, and leave the real data,
	 * then we step through each data-string in 2-byte blocks, convert the hex to ascii
	 * and write to file. */
	hex = malloc(sizeof(char) * 2);
	serresult = ser_send_ack(atcommand);
	for (counter = 0; counter < serresult.elemcount; counter++) {
		if (!strstr(serresult.result[counter], "^SBNR") && !strstr(serresult.result[counter], "OK")) {
			for (charc = 0; charc < strlen(serresult.result[counter]); charc+=2) {
				strncpy(hex, serresult.result[counter]+charc, 2);
				fprintf(fd, "%c", (int)strtol(hex, NULL, 16));
				bytecount++;
			}
		}
	}
	free(hex); free(atcommand); fclose(fd);
	printf("(%d bytes)\n", bytecount);
}

void sync_time (void) {
	struct s_result serresult;
	time_t seconds;
	char timestr[128];
	char *atcommand;

	atcommand = calloc(128,1);
	time(&seconds);
	/* parse current time to a format the GSM will eat */
	strftime(timestr,sizeof(timestr),"%y/%m/%d,%H:%M:%S",localtime(&seconds));
	/* construct and send the atcommand to set the time */
	sprintf(atcommand, "AT+CCLK=\"%s\"", timestr);
	serresult = ser_send_ack(atcommand);
	if (strcmp(serresult.result[0], "OK") == 0)
		printf("\nTime on GSM was synched to the system's clock (%s)\n\n", timestr);
	else
		printf("\nCouldn't sync time on GSM to system's clock!\n\n");
	free(atcommand);
}

void show_phonebook (const char *which) {
	struct s_result serresult;
	struct s_result rxpresult;
	int counter = 0;
	char *atcommand;

	/* Construct AT command to select the phonebook, then send that command. */
	atcommand = calloc(15, 1);
	sprintf(atcommand, "AT+CPBS=%s", which);
	serresult = ser_send_ack(atcommand);

	/* Figure out how many entries there are in this phonebook */
	serresult = ser_send_ack("AT+CPBS?");
	/* Match the last 2 digits (ammount of entries),(max entries) */
	rxpresult = rxp("(\\d+),(\\d+)$", serresult.result[0]);
	if (strcmp(rxpresult.result[1], "0") == 0) {
		printf("Empty List!\n");
		free(atcommand);
		return;
	}

	/* Ask for a dump of the possible entries */
	sprintf(atcommand, "AT+CPBR=1,%s", rxpresult.result[1]);
	serresult = ser_send_ack(atcommand);
	for (counter = 0; counter < serresult.elemcount-1; counter++) {
		/* One by one, match out all data, and display the useful stuff. */
		rxpresult = rxp("(\\d+),\"(.*)\",(\\d+),\"(.*)\"$", serresult.result[counter]);
		printf("%s. ", rxpresult.result[1]);
		/* If the namefield is empty, the number isn't programmed in the phone. */
		if (strcmp(rxpresult.result[4], "") == 0)
			printf("%-21s ", "Unknown");
		else
			printf("%-21s ", rxpresult.result[4]);
		printf("(%s)\n", rxpresult.result[2]);
	}
	free(atcommand);
}

void scan_networks (void) {
	struct s_result serresult;
	struct s_result rxpresult;
	int c;

	printf("\nScanning for networks... (takes a few secs)\n\n");
	serresult = ser_send_ack("AT+COPS=?");
	rxpresult = rxp("\\((\\d),\"(\\D+)\",,\"(\\d+)\"\\)", serresult.result[0]);
	for (c = 0; c < rxpresult.elemcount; c++) {
		int state = atoi(rxpresult.result[c+1]);
		printf(" * Network name:\t%s\n", rxpresult.result[c+2]);
		printf("   State:\t\t");
		switch (state) {
			case 1:
				printf("Selectable\n"); break;
			case 2:
				printf("Currently Selected\n"); break;
			case 3:
				printf("Emergency calls only\n"); break;
			default:
				printf("Unknown!\n"); break;
		}
		printf("   Operator ID:\t\t%s\n", rxpresult.result[c+3]);
		c += 3;
	}
}

void information (void) {
	struct s_result serresult;
	struct s_result rxpresult;
	int dBm = -113;

	printf("\n");
	serresult = ser_send_ack("AT+CGMI");
	printf("Manufacturer:\t\t%s\n", serresult.result[0]);
	serresult = ser_send_ack("AT+CGMM");
	printf("Model:\t\t\t%s\n", serresult.result[0]);
	serresult = ser_send_ack("AT+CGMR");
	printf("Firmware:\t\t%s\n", serresult.result[0]);
	serresult = ser_send_ack("AT+CIMI");
	printf("SIM Serial (IMSI):\t%s\n", serresult.result[0]);
	serresult = ser_send_ack("AT+CGSN");
	printf("Phone Serial (IMEI):\t%s\n", serresult.result[0]);
	serresult = ser_send_ack("AT^SCID");
	rxpresult = rxp("(\\d+)$", serresult.result[0]);
	printf("SIM-ID:\t\t\t%s\n", rxpresult.result[1]);
	serresult = ser_send_ack("AT+CBC");
	rxpresult = rxp("([0-9]+)$", serresult.result[0]);
	printf("Battery:\t\t%s%%\n", rxpresult.result[0]);
	serresult = ser_send_ack("AT+COPS?");
	rxpresult = rxp(",\"(.*)\"$", serresult.result[0]);
	printf("Registered network:\t%s\n", rxpresult.result[1]);
	serresult = ser_send_ack("AT+CSCA?");
	rxpresult = rxp("\"(.*)\",", serresult.result[0]);
	printf("SMS Center:\t\t%s\n", rxpresult.result[1]);
	serresult = ser_send_ack("AT+CSQ");
	rxpresult = rxp(": (\\d+),(\\d+)$", serresult.result[0]);
	printf("Signal Strength\t\t%d dBm (closer to zero is better)\n", dBm + (2 * atoi(rxpresult.result[1])));
}
